@@ -38,7 +38,6 @@ struct _MidoriPanel
     GtkWidget* toolbook;
     GtkWidget* notebook;
     GtkActionGroup* action_group;
-    GtkMenu*   menu;
 
     gboolean show_titles;
     gboolean show_controls;
@@ -63,7 +62,6 @@ enum
 
     PROP_SHADOW_TYPE,
     PROP_ACTION_GROUP,
-    PROP_MENU,
     PROP_PAGE,
     PROP_SHOW_TITLES,
     PROP_SHOW_CONTROLS,
@@ -165,24 +163,6 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      GTK_TYPE_ACTION_GROUP,
                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-    /**
-     * MidoriWebSettings:menu:
-     *
-     * This is the menu that holds the panel menu items.
-     *
-     * You shouldn't use this menu or add items.
-     *
-     * Deprecated: 0.1.9
-     */
-    g_object_class_install_property (gobject_class,
-                                     PROP_MENU,
-                                     g_param_spec_object (
-                                     "menu",
-                                     "Menu",
-                                     "Menu to hold panel items",
-                                     GTK_TYPE_MENU,
-                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
     g_object_class_install_property (gobject_class,
                                      PROP_PAGE,
                                      g_param_spec_int (
@@ -508,9 +488,6 @@ midori_panel_set_property (GObject*      object,
     case PROP_ACTION_GROUP:
         katze_object_assign (panel->action_group, g_value_dup_object (value));
         break;
-    case PROP_MENU:
-        katze_object_assign (panel->menu, g_value_dup_object (value));
-        break;
     case PROP_PAGE:
         midori_panel_set_current_page (panel, g_value_get_int (value));
         break;
@@ -556,9 +533,6 @@ midori_panel_get_property (GObject*    object,
     case PROP_ACTION_GROUP:
         g_value_set_object (value, panel->action_group);
         break;
-    case PROP_MENU:
-        g_value_set_object (value, panel->menu);
-        break;
     case PROP_PAGE:
         g_value_set_int (value, midori_panel_get_current_page (panel));
         break;
@@ -1150,3 +1124,14 @@ midori_panel_append_widget (MidoriPanel* panel,
                       G_CALLBACK (midori_panel_widget_destroy_cb), viewable);
     return midori_panel_append_page (panel, MIDORI_VIEWABLE (viewable));
 }
+
+/* Private function, used by MidoriBrowser */
+void
+midori_panel_set_toolbar_style (MidoriPanel*    panel,
+                                GtkToolbarStyle style)
+{
+    g_return_if_fail (MIDORI_IS_PANEL (panel));
+
+    gtk_toolbar_set_style (GTK_TOOLBAR (panel->toolbar), style);
+}
+
