@@ -46,6 +46,20 @@ midori_new_view_get_type (void) G_GNUC_CONST;
 
 #define MIDORI_TYPE_VIEW \
     (midori_view_get_type ())
+
+typedef enum
+{
+    MIDORI_SECURITY_NONE, /* The connection is neither encrypted nor verified. */
+    MIDORI_SECURITY_UNKNOWN, /* The security is unknown, due to lack of validation. */
+    MIDORI_SECURITY_TRUSTED /* The security is validated and trusted. */
+} MidoriSecurity;
+
+GType
+midori_security_get_type (void) G_GNUC_CONST;
+
+#define MIDORI_TYPE_SECURITY \
+    (midori_security_get_type ())
+
 #define MIDORI_VIEW(obj) \
     (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_VIEW, MidoriView))
 #define MIDORI_VIEW_CLASS(klass) \
@@ -61,7 +75,7 @@ typedef struct _MidoriView                MidoriView;
 typedef struct _MidoriViewClass           MidoriViewClass;
 
 GType
-midori_view_get_type                   (void);
+midori_view_get_type                   (void) G_GNUC_CONST;
 
 GtkWidget*
 midori_view_new                        (KatzeNet*          net);
@@ -92,6 +106,9 @@ midori_view_get_display_title          (MidoriView*        view);
 GdkPixbuf*
 midori_view_get_icon                   (MidoriView*        view);
 
+const gchar*
+midori_view_get_icon_uri               (MidoriView*        view);
+
 const gchar*
 midori_view_get_link_uri               (MidoriView*        view);
 
@@ -160,6 +177,12 @@ midori_view_can_go_forward             (MidoriView*        view);
 void
 midori_view_go_forward                 (MidoriView*        view);
 
+const gchar*
+midori_view_get_previous_page          (MidoriView*        view);
+
+const gchar*
+midori_view_get_next_page              (MidoriView*        view);
+
 gboolean
 midori_view_can_print                  (MidoriView*        view);
 
@@ -195,6 +218,23 @@ midori_view_execute_script             (MidoriView*        view,
                                         const gchar*       script,
                                         gchar**            exception);
 
+GdkPixbuf*
+midori_view_get_snapshot               (MidoriView*        view,
+                                        gint               width,
+                                        gint               height);
+
+GtkWidget*
+midori_view_get_web_view               (MidoriView*        view);
+
+MidoriSecurity
+midori_view_get_security               (MidoriView*        view);
+
+void
+midori_view_populate_popup             (MidoriView*        view,
+                                        GtkWidget*         menu,
+                                        gboolean           manual);
+
+
 G_END_DECLS
 
 #endif /* __MIDORI_VIEW_H__ */
