@@ -13,6 +13,14 @@
 
 #include <glib/gi18n.h>
 
+enum {
+    POPULATE_OPTION_MENU,
+
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 static void
 midori_viewable_base_init (MidoriViewableIface* iface);
 
@@ -68,6 +76,27 @@ midori_viewable_base_init (MidoriViewableIface* iface)
     if (initialized)
         return;
 
+    /**
+     * MidoriViewable::populate-option-menu:
+     * @viewable: the object on which the signal is emitted
+     * @menu: the #GtkMenu to populate
+     *
+     * Emitted when an Option menu is displayed, for instance
+     * when the user clicks the Options button in the panel.
+     *
+     * Deprecated: 0.2.3
+     */
+    signals[POPULATE_OPTION_MENU] = g_signal_new (
+        "populate-option-menu",
+        G_TYPE_FROM_INTERFACE (iface),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST),
+        0,
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__OBJECT,
+        G_TYPE_NONE, 1,
+        GTK_TYPE_MENU);
+
     iface->p = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
     iface->get_stock_id = midori_viewable_default_get_stock_id;
@@ -93,6 +122,8 @@ midori_viewable_base_finalize (MidoriViewableIface* iface)
  * the #MidoriViewable via midori_viewable_register_protocol().
  *
  * Return value: a new #MidoriViewable, or %NULL
+ *
+ * Deprecated: 0.2.6
  **/
 GtkWidget*
 midori_viewable_new_from_uri (const gchar* uri)
@@ -181,6 +212,8 @@ viewable_type_implements (GType type,
  * "midori://dummy": support URIs like "midori://dummy/foo"
  *
  * Return value: a new #MidoriViewable, or %NULL
+ *
+ * Deprecated: 0.2.6
  **/
 void
 midori_viewable_register_protocol (GType        type,
@@ -258,6 +291,6 @@ midori_viewable_get_toolbar (MidoriViewable* viewable)
 
     toolbar = MIDORI_VIEWABLE_GET_IFACE (viewable)->get_toolbar (viewable);
     if (!toolbar)
-        toolbar = gtk_event_box_new ();
+        toolbar = gtk_toolbar_new ();
     return toolbar;
 }
