@@ -12,6 +12,7 @@
 #include "midori-history.h"
 
 #include "midori-app.h"
+#include "midori-array.h"
 #include "midori-browser.h"
 #include "midori-stock.h"
 #include "midori-view.h"
@@ -23,22 +24,16 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
-#include <katze/katze.h>
 #include <gdk/gdkkeysyms.h>
 
+#define COMPLETION_DELAY 200
+
 void
 midori_browser_edit_bookmark_dialog_new (MidoriBrowser* browser,
                                          KatzeItem*     bookmark,
                                          gboolean       new_bookmark,
                                          gboolean       is_folder);
 
-#include "config.h"
-
-#if HAVE_SQLITE
-    #include <sqlite3.h>
-#endif
-
-#define COMPLETION_DELAY 150
 
 struct _MidoriHistory
 {
@@ -125,7 +120,40 @@ midori_history_get_stock_id (MidoriViewable* viewable)
     return STOCK_HISTORY;
 }
 
-#if HAVE_SQLITE
+static gchar*
+midori_history_format_date (KatzeItem *item)
+{
+    gint age;
+    gint64 day;
+    gchar token[50];
+    gchar* sdate;
+    time_t current_time;
+
+    current_time = time (NULL);
+    day = katze_item_get_added (item);
+
+    age = sokoke_days_between ((time_t*)&day, &current_time);
+
+    /* A negative age is a date in the future, the clock is probably off */
+    if (age < -1)
+        ;
+    else if (age > 7 || age < 0)
+    {
+        strftime (token, sizeof (token), "%x", localtime ((time_t*)&day));
+        sdate = g_strdup (token);
+    }
+    else if (age > 6)
+        sdate = _("A week ago");
+    else if (age > 1)
+        sdate = g_strdup_printf (ngettext ("%d day ago",
+            "%d days ago", (gint)age), (gint)age);
+    else if (age == 0)
+        sdate = _("Today");
+    else
+        sdate = _("Yesterday");
+    return sdate;
+}
+
 static void
 midori_history_clear_db (MidoriHistory* history)
 {
@@ -155,7 +183,7 @@ midori_history_remove_item_from_db (MidoriHistory* history,
 
     db = g_object_get_data (G_OBJECT (history->array), "db");
 
-    if (katze_item_get_uri (item))
+    if (KATZE_ITEM_IS_BOOKMARK (item))
         sqlcmd = sqlite3_mprintf (
             "DELETE FROM history WHERE uri = '%q' AND"
             " title = '%q' AND date = %llu",
@@ -163,8 +191,8 @@ midori_history_remove_item_from_db (MidoriHistory* history,
             katze_item_get_name (item),
             katze_item_get_added (item));
     else
-       sqlcmd = sqlite3_mprintf (
-            "DELETE FROM history WHERE day = %d", katze_item_get_added (item));
+       sqlcmd = sqlite3_mprintf ("DELETE FROM history WHERE day = %d",
+                katze_item_get_meta_integer (item, "day"));
 
     if (sqlite3_exec (db, sqlcmd, NULL, NULL, &errmsg) != SQLITE_OK)
     {
@@ -178,8 +206,6 @@ midori_history_remove_item_from_db (MidoriHistory* history,
 /**
  * midori_history_read_from_db:
  * @history: a #MidoriHistory
- * @model: a #GtkTreeStore
- * @parent: a #GtkTreeIter, or %NULL
  * @req_day: the timestamp of one day, or 0
  * @filter: a filter string to search for
  *
@@ -188,10 +214,8 @@ midori_history_remove_item_from_db (MidoriHistory* history,
  * 2. If @req_day is given, all pages for that day are added.
  * 3. If @filter is given, all pages matching the filter are added.
  **/
-static gboolean
+static KatzeArray*
 midori_history_read_from_db (MidoriHistory* history,
-                             GtkTreeStore*  model,
-                             GtkTreeIter*   parent,
                              int            req_day,
                              const gchar*   filter)
 {
@@ -199,10 +223,6 @@ midori_history_read_from_db (MidoriHistory* history,
     sqlite3_stmt* statement;
     gint result;
     const gchar* sqlcmd;
-    time_t current_time;
-
-    GtkTreeIter iter;
-    GtkTreeIter root_iter;
 
     db = g_object_get_data (G_OBJECT (history->array), "db");
 
@@ -210,16 +230,17 @@ midori_history_read_from_db (MidoriHistory* history,
     {
         gchar* filterstr;
 
-        sqlcmd = "SELECT uri, title, day FROM history_view "
-                 "WHERE uri LIKE ? or title LIKE ? GROUP BY uri "
+        sqlcmd = "SELECT * FROM ("
+                 "    SELECT uri, title, day FROM history"
+                 "    WHERE uri LIKE ?1 OR title LIKE ?1 GROUP BY uri "
                  "UNION ALL "
-                 "SELECT replace(uri, '%s', title) AS uri, title, day "
-                 "FROM search_view WHERE title LIKE ?1 GROUP BY uri "
-                 "ORDER BY day ASC";
+                 "    SELECT replace (uri, '%s', keywords) AS uri, "
+                 "    keywords AS title, day FROM search "
+                 "    WHERE uri LIKE ?1 OR keywords LIKE ?1 GROUP BY uri "
+                 ") ORDER BY day ASC";
         result = sqlite3_prepare_v2 (db, sqlcmd, -1, &statement, NULL);
         filterstr = g_strdup_printf ("%%%s%%", filter);
         sqlite3_bind_text (statement, 1, filterstr, -1, g_free);
-        sqlite3_bind_text (statement, 2, g_strdup (filterstr), -1, g_free);
         req_day = -1;
     }
     else if (req_day == 0)
@@ -237,111 +258,36 @@ midori_history_read_from_db (MidoriHistory* history,
     }
 
     if (result != SQLITE_OK)
-        return FALSE;
+        return NULL;
 
-    if (req_day == 0)
-        current_time = time (NULL);
-
-    while ((result = sqlite3_step (statement)) == SQLITE_ROW)
-    {
-        KatzeItem* item;
-        const unsigned char* uri;
-        const unsigned char* title;
-        sqlite3_int64 date;
-        sqlite3_int64 day;
-
-        if (req_day == 0)
-        {
-            gint age;
-            gchar token[50];
-            gchar* sdate;
-
-            day = sqlite3_column_int64 (statement, 0);
-            date = sqlite3_column_int64 (statement, 1);
-
-            item = katze_item_new ();
-            katze_item_set_added (item, day);
-            age = sokoke_days_between ((time_t*)&date, &current_time);
-
-            /* A negative age is a date in the future, the clock is probably off */
-            if (age < -1)
-            {
-                static gboolean clock_warning = FALSE;
-                if (!clock_warning)
-                {
-                    midori_app_send_notification (history->app,
-                        _("Erroneous clock time"),
-                        _("The clock time lies in the past. "
-                          "Please check the current date and time."));
-                    clock_warning = TRUE;
-                }
-            }
-
-            if (age > 7 || age < 0)
-            {
-                strftime (token, sizeof (token), "%x", localtime ((time_t*)&date));
-                sdate = token;
-            }
-            else if (age > 6)
-                sdate = _("A week ago");
-            else if (age > 1)
-                sdate = g_strdup_printf (ngettext ("%d day ago",
-                    "%d days ago", (gint)age), (gint)age);
-            else if (age == 0)
-                sdate = _("Today");
-            else
-                sdate = _("Yesterday");
-
-            gtk_tree_store_insert_with_values (model, &root_iter, NULL,
-                                               0, 0, item, 1, sdate, -1);
-            /* That's an invisible dummy, so we always have an expander */
-            gtk_tree_store_insert_with_values (model, &iter, &root_iter,
-                0, 0, NULL, 1, NULL, -1);
-
-            if (age > 1 && age < 7)
-                g_free (sdate);
-        }
-        else
-        {
-            uri = sqlite3_column_text (statement, 0);
-            title = sqlite3_column_text (statement, 1);
-            date = sqlite3_column_int64 (statement, 2);
-            day = sqlite3_column_int64 (statement, 3);
-            if (!uri)
-                continue;
-
-            item = katze_item_new ();
-            katze_item_set_added (item, date);
-            katze_item_set_uri (item, (gchar*)uri);
-            katze_item_set_name (item, (gchar*)title);
-            gtk_tree_store_insert_with_values (model, NULL, parent,
-                0, 0, item, 1, katze_item_get_name (item), -1);
-        }
-    }
-
-    if (req_day != 0 && !(filter && *filter))
-    {
-        /* Remove invisible dummy row */
-        GtkTreeIter child;
-        gint last = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), parent);
-        gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (model), &child, parent, last - 1);
-        gtk_tree_store_remove (model, &child);
-    }
-
-    if (result != SQLITE_DONE)
-        g_print (_("Failed to execute database statement: %s\n"),
-                 sqlite3_errmsg (db));
-
-    sqlite3_finalize (statement);
-    return FALSE;
+    return katze_array_from_statement (statement);
 }
 
 static void
-midori_history_add_clicked_cb (GtkWidget* toolitem)
+midori_history_read_from_db_to_model (MidoriHistory* history,
+                                      GtkTreeStore*  model,
+                                      GtkTreeIter*   parent,
+                                      int            req_day,
+                                      const gchar*   filter)
 {
-    MidoriBrowser* browser = midori_browser_get_for_widget (toolitem);
-    /* FIXME: Take selected folder into account */
-    midori_browser_edit_bookmark_dialog_new (browser, NULL, TRUE, FALSE);
+    KatzeArray* array;
+    gint last;
+    KatzeItem* item;
+    GtkTreeIter child;
+
+    array = midori_history_read_from_db (history, req_day, filter);
+    katze_bookmark_populate_tree_view (array, model, parent);
+
+    /* Remove invisible dummy row */
+    last = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), parent);
+    if (!last)
+        return;
+    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (model), &child, parent, last - 1);
+    gtk_tree_model_get (GTK_TREE_MODEL (model), &child, 0, &item, -1);
+    if (KATZE_ITEM_IS_SEPARATOR (item))
+        gtk_tree_store_remove (model, &child);
+    else
+        g_object_unref (item);
 }
 
 static void
@@ -385,36 +331,25 @@ midori_history_clear_clicked_cb (GtkWidget*     toolitem,
 }
 
 static void
-midori_history_cursor_or_row_changed_cb (GtkTreeView*   treeview,
-                                         MidoriHistory* history)
+midori_history_bookmark_add_cb (GtkWidget*     menuitem,
+                                MidoriHistory* history)
 {
     GtkTreeModel* model;
     GtkTreeIter iter;
     KatzeItem* item;
 
-    if (!history->bookmark)
-        return;
-
-    if (katze_tree_view_get_selected_iter (treeview, &model, &iter))
-    {
-        gboolean is_page;
-
+    MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (history));
+    if (katze_tree_view_get_selected_iter (GTK_TREE_VIEW (history->treeview),
+                                           &model, &iter))
         gtk_tree_model_get (model, &iter, 0, &item, -1);
 
-        is_page = item && katze_item_get_uri (item);
-        gtk_widget_set_sensitive (history->bookmark, is_page);
-        gtk_widget_set_sensitive (history->delete, TRUE);
-
-        if (item)
-            g_object_unref (item);
-    }
+    if (KATZE_IS_ITEM (item) && katze_item_get_uri (item))
+        midori_browser_edit_bookmark_dialog_new (browser, item, TRUE, FALSE);
     else
-    {
-        gtk_widget_set_sensitive (history->bookmark, FALSE);
-        gtk_widget_set_sensitive (history->delete, FALSE);
-    }
+        midori_browser_edit_bookmark_dialog_new (browser, NULL, TRUE, FALSE);
+
+    g_object_unref (item);
 }
-#endif
 
 static GtkWidget*
 midori_history_get_toolbar (MidoriViewable* viewable)
@@ -424,20 +359,17 @@ midori_history_get_toolbar (MidoriViewable* viewable)
     if (!history->toolbar)
     {
         GtkWidget* toolbar;
-        #if HAVE_SQLITE
         GtkToolItem* toolitem;
-        #endif
 
         toolbar = gtk_toolbar_new ();
         gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_BUTTON);
         history->toolbar = toolbar;
-        #if HAVE_SQLITE
         toolitem = gtk_tool_button_new_from_stock (STOCK_BOOKMARK_ADD);
         gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
                                      _("Bookmark the selected history item"));
         gtk_tool_item_set_is_important (toolitem, TRUE);
         g_signal_connect (toolitem, "clicked",
-            G_CALLBACK (midori_history_add_clicked_cb), history);
+            G_CALLBACK (midori_history_bookmark_add_cb), history);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
         gtk_widget_show (GTK_WIDGET (toolitem));
         history->bookmark = GTK_WIDGET (toolitem);
@@ -457,15 +389,12 @@ midori_history_get_toolbar (MidoriViewable* viewable)
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
         gtk_widget_show (GTK_WIDGET (toolitem));
         history->clear = GTK_WIDGET (toolitem);
-        midori_history_cursor_or_row_changed_cb (
-            GTK_TREE_VIEW (history->treeview), history);
         g_signal_connect (history->bookmark, "destroy",
             G_CALLBACK (gtk_widget_destroyed), &history->bookmark);
         g_signal_connect (history->delete, "destroy",
             G_CALLBACK (gtk_widget_destroyed), &history->delete);
         g_signal_connect (history->clear, "destroy",
             G_CALLBACK (gtk_widget_destroyed), &history->clear);
-        #endif
     }
 
     return history->toolbar;
@@ -479,6 +408,51 @@ midori_history_viewable_iface_init (MidoriViewableIface* iface)
     iface->get_toolbar = midori_history_get_toolbar;
 }
 
+static void
+midori_history_add_item_cb (KatzeArray*    array,
+                            KatzeItem*     item,
+                            MidoriHistory* history)
+{
+    GtkTreeView* treeview = GTK_TREE_VIEW (history->treeview);
+    GtkTreeModel* model = gtk_tree_view_get_model (treeview);
+    GtkTreeIter iter;
+    KatzeItem* today;
+    time_t current_time;
+
+    current_time = time (NULL);
+    if (gtk_tree_model_iter_children (model, &iter, NULL))
+    {
+        gint64 day;
+        gboolean has_today;
+
+        gtk_tree_model_get (model, &iter, 0, &today, -1);
+
+        day = katze_item_get_added (today);
+        has_today = sokoke_days_between ((time_t*)&day, &current_time) == 0;
+        g_object_unref (today);
+        if (has_today)
+        {
+            gchar* tooltip = g_markup_escape_text (katze_item_get_uri (item), -1);
+            KatzeItem* copy = katze_item_copy (item);
+            gtk_tree_store_insert_with_values (GTK_TREE_STORE (model), NULL, &iter,
+                                               0, 0, copy, 1, tooltip, -1);
+            g_object_unref (copy);
+            g_free (tooltip);
+            return;
+        }
+    }
+
+    today = (KatzeItem*)katze_array_new (KATZE_TYPE_ITEM);
+    katze_item_set_added (today, current_time);
+    katze_item_set_meta_integer (today, "day",
+                                 sokoke_time_t_to_julian (&current_time));
+    gtk_tree_store_insert_with_values (GTK_TREE_STORE (model), &iter, NULL,
+                                       0, 0, today, -1);
+    /* That's an invisible dummy, so we always have an expander */
+    gtk_tree_store_insert_with_values (GTK_TREE_STORE (model), NULL, &iter,
+                                       0, 0, NULL, -1);
+}
+
 static void
 midori_history_set_app (MidoriHistory* history,
                         MidoriApp*     app)
@@ -487,7 +461,9 @@ midori_history_set_app (MidoriHistory* history,
 
     if (history->array)
     {
-        g_object_unref (history->array);
+        g_signal_handlers_disconnect_by_func (history->array,
+            midori_history_add_item_cb, history);
+        katze_object_assign (history->array, NULL);
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
         gtk_tree_store_clear (GTK_TREE_STORE (model));
     }
@@ -498,11 +474,11 @@ midori_history_set_app (MidoriHistory* history,
     g_object_ref (app);
 
     history->array = katze_object_get_object (app, "history");
+    g_signal_connect (history->array, "add-item",
+                      G_CALLBACK (midori_history_add_item_cb), history);
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
-    #if HAVE_SQLITE
     if (history->array)
-        midori_history_read_from_db (history, GTK_TREE_STORE (model), NULL, 0, NULL);
-    #endif
+        midori_history_read_from_db_to_model (history, GTK_TREE_STORE (model), NULL, 0, NULL);
 }
 
 static void
@@ -572,36 +548,60 @@ midori_history_treeview_render_icon_cb (GtkTreeViewColumn* column,
     }
 }
 
-#if HAVE_SQLITE
+static void
+midori_history_treeview_render_text_cb (GtkTreeViewColumn* column,
+                                        GtkCellRenderer*   renderer,
+                                        GtkTreeModel*      model,
+                                        GtkTreeIter*       iter,
+                                        GtkWidget*         treeview)
+{
+    KatzeItem* item;
+
+    gtk_tree_model_get (model, iter, 0, &item, -1);
+
+    if (KATZE_ITEM_IS_BOOKMARK (item))
+        g_object_set (renderer, "markup", NULL,
+                      "text", katze_item_get_name (item), NULL);
+    else if (KATZE_ITEM_IS_FOLDER (item))
+        g_object_set (renderer, "markup", NULL,
+                      "text", midori_history_format_date (item), NULL);
+    else
+        g_object_set (renderer, "markup", _("<i>Separator</i>"), NULL);
+
+    if (item)
+        g_object_unref (item);
+}
+
 static void
 midori_history_row_activated_cb (GtkTreeView*       treeview,
-                                   GtkTreePath*       path,
-                                   GtkTreeViewColumn* column,
-                                   MidoriHistory*   history)
+                                 GtkTreePath*       path,
+                                 GtkTreeViewColumn* column,
+                                 MidoriHistory*     history)
 {
     GtkTreeModel* model;
     GtkTreeIter iter;
     KatzeItem* item;
-    const gchar* uri;
 
     model = gtk_tree_view_get_model (treeview);
 
     if (gtk_tree_model_get_iter (model, &iter, path))
     {
         gtk_tree_model_get (model, &iter, 0, &item, -1);
-
-        if (!item)
-            return;
-
-        uri = katze_item_get_uri (item);
-        if (uri && *uri)
+        if (KATZE_ITEM_IS_BOOKMARK (item))
         {
             MidoriBrowser* browser;
+            const gchar* uri;
 
+            uri = katze_item_get_uri (item);
             browser = midori_browser_get_for_widget (GTK_WIDGET (history));
             midori_browser_set_current_uri (browser, uri);
+            g_object_unref (item);
+            return;
         }
-
+        if (gtk_tree_view_row_expanded (treeview, path))
+            gtk_tree_view_collapse_row (treeview, path);
+        else
+            gtk_tree_view_expand_row (treeview, path, FALSE);
         g_object_unref (item);
     }
 }
@@ -624,9 +624,8 @@ midori_history_popup_item (GtkWidget*     menu,
         gtk_label_set_text_with_mnemonic (GTK_LABEL (gtk_bin_get_child (
         GTK_BIN (menuitem))), label);
     if (!strcmp (stock_id, GTK_STOCK_EDIT))
-        gtk_widget_set_sensitive (menuitem,
-            KATZE_IS_ARRAY (item) || uri != NULL);
-    else if (!KATZE_IS_ARRAY (item) && strcmp (stock_id, GTK_STOCK_DELETE))
+        gtk_widget_set_sensitive (menuitem, uri != NULL);
+    else if (katze_item_get_uri (item) && strcmp (stock_id, GTK_STOCK_DELETE))
         gtk_widget_set_sensitive (menuitem, uri != NULL);
     g_object_set_data (G_OBJECT (menuitem), "KatzeItem", item);
     g_signal_connect (menuitem, "activate", G_CALLBACK (callback), history);
@@ -660,26 +659,31 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
     guint n;
 
     item = (KatzeItem*)g_object_get_data (G_OBJECT (menuitem), "KatzeItem");
-    if (KATZE_IS_ARRAY (item))
+    if (KATZE_ITEM_IS_FOLDER (item))
     {
+        sqlite3* db;
+        gchar* sqlcmd;
         KatzeItem* child;
+        KatzeArray* array;
         guint i = 0;
 
-        while ((child = katze_array_get_nth_item (KATZE_ARRAY (item), i)))
+        db = g_object_get_data (G_OBJECT (history->array), "db");
+        sqlcmd = g_strdup_printf ("SELECT uri, title, date, day "
+                 "FROM history WHERE day = %d "
+                 "GROUP BY uri ORDER BY date ASC",
+                 (int)katze_item_get_added (item));
+        array = katze_array_from_sqlite (db, sqlcmd);
+        g_free (sqlcmd);
+        while ((child = katze_array_get_nth_item (KATZE_ARRAY (array), i++)))
         {
             if ((uri = katze_item_get_uri (child)) && *uri)
             {
                 MidoriBrowser* browser;
-                MidoriWebSettings* settings;
 
                 browser = midori_browser_get_for_widget (GTK_WIDGET (history));
                 n = midori_browser_add_item (browser, child);
-                settings = katze_object_get_object (browser, "settings");
-                if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
-                    midori_browser_set_current_page (browser, n);
-                g_object_unref (settings);
+                midori_browser_set_current_page_smartly (browser, n);
             }
-            i++;
         }
     }
     else
@@ -687,14 +691,10 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
         if ((uri = katze_item_get_uri (item)) && *uri)
         {
             MidoriBrowser* browser;
-            MidoriWebSettings* settings;
 
             browser = midori_browser_get_for_widget (GTK_WIDGET (history));
             n = midori_browser_add_item (browser, item);
-            settings = katze_object_get_object (browser, "settings");
-            if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
-                midori_browser_set_current_page (browser, n);
-            g_object_unref (settings);
+            midori_browser_set_current_page_smartly (browser, n);
         }
     }
 }
@@ -718,22 +718,6 @@ midori_history_open_in_window_activate_cb (GtkWidget*     menuitem,
     }
 }
 
-static void
-midori_history_bookmark_activate_cb (GtkWidget*     menuitem,
-                                     MidoriHistory* history)
-{
-    KatzeItem* item;
-    const gchar* uri;
-
-    item = (KatzeItem*)g_object_get_data (G_OBJECT (menuitem), "KatzeItem");
-    uri = katze_item_get_uri (item);
-
-    if (uri && *uri)
-    {
-        MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (history));
-        midori_browser_edit_bookmark_dialog_new (browser, item, TRUE, FALSE);
-    }
-}
 
 static void
 midori_history_popup (GtkWidget*      widget,
@@ -745,7 +729,7 @@ midori_history_popup (GtkWidget*      widget,
     GtkWidget* menuitem;
 
     menu = gtk_menu_new ();
-    if (KATZE_IS_ARRAY (item))
+    if (!katze_item_get_uri (item))
         midori_history_popup_item (menu,
             STOCK_TAB_NEW, _("Open all in _Tabs"),
             item, midori_history_open_in_tab_activate_cb, history);
@@ -758,7 +742,7 @@ midori_history_popup (GtkWidget*      widget,
         midori_history_popup_item (menu, STOCK_WINDOW_NEW, _("Open in New _Window"),
             item, midori_history_open_in_window_activate_cb, history);
         midori_history_popup_item (menu, STOCK_BOOKMARK_ADD, NULL,
-            item, midori_history_bookmark_activate_cb, history);
+            item, midori_history_bookmark_add_cb, history);
     }
     menuitem = gtk_separator_menu_item_new ();
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -863,8 +847,8 @@ midori_history_row_expanded_cb (GtkTreeView*   treeview,
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
     gtk_tree_model_get (model, iter, 0, &item, -1);
-    midori_history_read_from_db (history, GTK_TREE_STORE (model),
-                                 iter, katze_item_get_added (item), NULL);
+    midori_history_read_from_db_to_model (history, GTK_TREE_STORE (model),
+                                         iter, katze_item_get_meta_integer (item, "day"), NULL);
     g_object_unref (item);
 }
 
@@ -884,7 +868,7 @@ midori_history_row_collapsed_cb (GtkTreeView *treeview,
         gtk_tree_store_remove (treestore, &child);
     /* That's an invisible dummy, so we always have an expander */
     gtk_tree_store_insert_with_values (treestore, &child, parent,
-        0, 0, NULL, 1, NULL, -1);
+        0, 0, NULL, -1);
 }
 
 static gboolean
@@ -898,7 +882,7 @@ midori_history_filter_timeout_cb (gpointer data)
     treestore = GTK_TREE_STORE (model);
 
     gtk_tree_store_clear (treestore);
-    midori_history_read_from_db (history, treestore, NULL, 0, history->filter);
+    midori_history_read_from_db_to_model (history, treestore, NULL, 0, history->filter);
 
     return FALSE;
 }
@@ -913,7 +897,7 @@ midori_history_filter_entry_changed_cb (GtkEntry*      entry,
         midori_history_filter_timeout_cb, history);
     katze_assign (history->filter, g_strdup (gtk_entry_get_text (entry)));
 }
-#endif
+
 static void
 midori_history_filter_entry_clear_cb (GtkEntry*      entry,
                                       gint           icon_pos,
@@ -939,17 +923,19 @@ midori_history_init (MidoriHistory* history)
     /* Create the filter entry */
     entry = gtk_icon_entry_new ();
     gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-        GTK_ICON_ENTRY_SECONDARY, GTK_STOCK_CLEAR);
+                                        GTK_ICON_ENTRY_PRIMARY,
+                                        GTK_STOCK_FIND);
+    gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
+                                        GTK_ICON_ENTRY_SECONDARY,
+                                        GTK_STOCK_CLEAR);
     gtk_icon_entry_set_icon_highlight (GTK_ICON_ENTRY (entry),
-        GTK_ICON_ENTRY_SECONDARY, TRUE);
+                                       GTK_ICON_ENTRY_SECONDARY,
+                                       TRUE);
     g_signal_connect (entry, "icon-release",
         G_CALLBACK (midori_history_filter_entry_clear_cb), history);
-    #if HAVE_SQLITE
     g_signal_connect (entry, "changed",
         G_CALLBACK (midori_history_filter_entry_changed_cb), history);
-    #endif
     box = gtk_hbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (box), gtk_label_new (_("Filter:")), FALSE, FALSE, 3);
     gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 3);
     gtk_widget_show_all (box);
     gtk_box_pack_start (GTK_BOX (history), box, FALSE, FALSE, 5);
@@ -958,6 +944,7 @@ midori_history_init (MidoriHistory* history)
     model = gtk_tree_store_new (2, KATZE_TYPE_ITEM, G_TYPE_STRING);
     treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
+    gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (treeview), 1);
     column = gtk_tree_view_column_new ();
     renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, renderer_pixbuf, FALSE);
@@ -966,18 +953,14 @@ midori_history_init (MidoriHistory* history)
         treeview, NULL);
     renderer_text = gtk_cell_renderer_text_new ();
     gtk_tree_view_column_pack_start (column, renderer_text, FALSE);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (column), renderer_text,
-        "text", 1, NULL);
+    gtk_tree_view_column_set_cell_data_func (column, renderer_text,
+        (GtkTreeCellDataFunc)midori_history_treeview_render_text_cb,
+        treeview, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
     g_object_unref (model);
-    #if HAVE_SQLITE
     g_object_connect (treeview,
                       "signal::row-activated",
                       midori_history_row_activated_cb, history,
-                      "signal::cursor-changed",
-                      midori_history_cursor_or_row_changed_cb, history,
-                      "signal::columns-changed",
-                      midori_history_cursor_or_row_changed_cb, history,
                       "signal::button-release-event",
                       midori_history_button_release_event_cb, history,
                       "signal::key-release-event",
@@ -989,7 +972,6 @@ midori_history_init (MidoriHistory* history)
                       "signal::popup-menu",
                       midori_history_popup_menu_cb, history,
                       NULL);
-    #endif
     gtk_widget_show (treeview);
     gtk_box_pack_start (GTK_BOX (history), treeview, TRUE, TRUE, 0);
     history->treeview = treeview;
@@ -1006,8 +988,8 @@ midori_history_finalize (GObject* object)
     if (history->app)
         g_object_unref (history->app);
 
-    /* FIXME: We don't unref items (last argument is FALSE) because
-       our reference counting is incorrect. */
+    g_signal_handlers_disconnect_by_func (history->array,
+        midori_history_add_item_cb, history);
     g_object_unref (history->array);
     katze_assign (history->filter, NULL);
 }
